import { __awaiter } from 'tslib';
import { ɵɵdefineInjectable, ɵɵinject, Injectable } from '@angular/core';
import { DcSccBaseFactory, DcScc } from 'dc-scc';
import { DcEasApi } from 'dc-eas-api';
import { DcDdsObjectModelV1 } from 'dc-dds-api';

class BaseFactory extends DcSccBaseFactory {
    newRequest() {
        return __awaiter(this, void 0, void 0, function* () {
            var reqObj = yield this.getService().newRequest();
            return reqObj;
        });
    }
    clearCache(obj) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof this._s[obj.getGuid()] != "undefined") {
                delete this._s[obj.getGuid()];
            }
            return this;
        });
    }
    getByGuids(guids, cbObj = null, cbMethod = null, cbRef = null) {
        return __awaiter(this, void 0, void 0, function* () {
            var utilObj = this.getService().getDcScc().getUtilities().getFactories().getAsyncGuids();
            return yield utilObj.getByGuids(guids, this, "getByGuid", cbObj, cbMethod, cbRef);
        });
    }
}

class Connection extends BaseFactory {
    constructor() {
        super(...arguments);
        this._wsHost = "dc-wss-nlb.datacamo.net";
        this._wsPort = 28109;
        this._wsSsl = true;
        this._isReconn = false;
        this._wrkObj = null;
    }
    setConfig(host, port, ssl = true) {
        this._wsHost = host;
        this._wsPort = port;
        this._wsSsl = ssl;
        return this;
    }
    getClient() {
        return __awaiter(this, void 0, void 0, function* () {
            var wrkObj = yield this.getWorker();
            return wrkObj.getClient();
        });
    }
    getWorker() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._wrkObj === null) {
                try {
                    this._wrkObj = this.getService().getDcScc().getWorkers().getV3(this._wsHost, this._wsPort, this._wsSsl);
                    yield this._wrkObj.initialize();
                    var cObj = this._wrkObj.getClient();
                    var reqObj = cObj.newMessage();
                    reqObj.setL1("Authentication").setL2("Get").setL3("Salt");
                    var salt = yield reqObj.getResponse(true);
                    var reqObj = cObj.newMessage();
                    reqObj.setL1("Authentication").setL2("Validate").setL3("ByEAS").setL4("EphemeralKey");
                    var hash = yield this.getMtm().get("utilities").getHashing().getSHA().get256("UNR--" + salt);
                    var ephKey = yield this.getMtm().get("DcEasApi").getEphemeralKeys().newReadUserPublic();
                    yield ephKey.signRequest(reqObj, hash);
                    var sessObj = yield reqObj.getResponse(true);
                    if (typeof sessObj["session-id"] != "undefined"
                        && typeof sessObj["session-key"] != "undefined") {
                        cObj.setSession(sessObj["session-id"], sessObj["session-key"]);
                        this._wrkObj.setTermCb(this, "reconnect");
                    }
                    else {
                        throw this.getMtm().newException("Server authentication returned an invalid session");
                    }
                }
                catch (errObj) {
                    this._wrkObj = null;
                    throw errObj;
                }
            }
            return this._wrkObj;
        });
    }
    reconnect(wrkObj) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._isReconn === false) {
                this._isReconn = true;
                this.logInfo("UNR Reconnecting");
                try {
                    try {
                        this._wrkObj = null;
                        yield this.getWorker();
                        this._isReconn = false;
                        this.logInfo("UNR Reconnect success");
                    }
                    catch (errObj) {
                        throw this.getService().getMtm().newException("UNR Reconnect failed");
                    }
                }
                catch (errObj) {
                    this._isReconn = false;
                    this.callException(errObj);
                }
            }
        });
    }
}

class Base extends DcDdsObjectModelV1 {
    constructor() {
        super(...arguments);
        this._guid = null;
    }
    baseConstruct(apiObj) {
        return __awaiter(this, void 0, void 0, function* () {
            this.setExceptionCb(this.getService().getLogging(), "exception");
            this._guid = apiObj.guid;
            if (typeof apiObj["ddsId"] != "undefined") {
                var ddsObj = yield this.getService().getDdsClient();
                this.setConfig(ddsObj, apiObj.ddsId, apiObj.ddsAuth);
                var storeObj = yield this.pubStore();
                yield storeObj.initialize();
            }
        });
    }
    getGuid() {
        return this._guid;
    }
    newRequest() {
        return __awaiter(this, void 0, void 0, function* () {
            var reqObj = yield this.getService().newRequest();
            return reqObj;
        });
    }
}

class Alpha extends Base {
    constructor() {
        super(...arguments);
        this._nsGuids = null;
        this._mgmGuids = null;
    }
    construct(apiObj) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.baseConstruct(apiObj);
            var storeObj = yield this.pubStore();
            storeObj.addUpdateCb(this, "ddsUpdateCb");
            this.setInit();
            return this;
        });
    }
    getDisplayName() {
        return this.getPubKey("displayName", true);
    }
    getCreated() {
        return this.getPubKey("created", true);
    }
}

class DdsSync extends Alpha {
    ddsUpdateCb(storeObj) {
        this._nsGuids = null;
        this._mgmGuids = null;
    }
}

class Privileges extends DdsSync {
    //root space will always match since it returns "";
    getNsMaps() { }
    ;
    getMgmMaps() { }
    ;
    canReadNsByStr(nsStr, throwOnErr = false) {
        return __awaiter(this, void 0, void 0, function* () {
            var roleFact = this.getService().getRoles();
            var mapObjs = yield this.getNsMaps();
            for (var i in mapObjs) {
                var mapObj = mapObjs[i];
                if ((yield roleFact.canRead(yield mapObj.getRole(), false)) === true) {
                    var nsObj = yield mapObj.getNamespace();
                    var exStr = yield nsObj.getExtendedNamespace();
                    if (nsStr === exStr || nsStr.indexOf(exStr) === 0) {
                        return true;
                    }
                }
            }
            if (throwOnErr === true) {
                throw this.getMtm().newException("User is not allowed to read the namespace", 19556);
            }
            else {
                return false;
            }
        });
    }
    canReadSensitiveNsByStr(nsStr, throwOnErr = false) {
        return __awaiter(this, void 0, void 0, function* () {
            var roleFact = this.getService().getRoles();
            var mapObjs = yield this.getNsMaps();
            for (var i in mapObjs) {
                var mapObj = mapObjs[i];
                if ((yield roleFact.canReadSensitive(yield mapObj.getRole(), false)) === true) {
                    var nsObj = yield mapObj.getNamespace();
                    var exStr = yield nsObj.getExtendedNamespace();
                    if (nsStr === exStr || nsStr.indexOf(exStr) === 0) {
                        return true;
                    }
                }
            }
            if (throwOnErr === true) {
                throw this.getMtm().newException("User is not allowed to read sensitive in the namespace", 19557);
            }
            else {
                return false;
            }
        });
    }
    canExecuteNsByStr(nsStr, throwOnErr = false) {
        return __awaiter(this, void 0, void 0, function* () {
            var roleFact = this.getService().getRoles();
            var mapObjs = yield this.getNsMaps();
            for (var i in mapObjs) {
                var mapObj = mapObjs[i];
                if ((yield roleFact.canExecute(yield mapObj.getRole(), false)) === true) {
                    var nsObj = yield mapObj.getNamespace();
                    var exStr = yield nsObj.getExtendedNamespace();
                    if (nsStr === exStr || nsStr.indexOf(exStr) === 0) {
                        return true;
                    }
                }
            }
            if (throwOnErr === true) {
                throw this.getMtm().newException("User is not allowed to execute in the namespace", 19558);
            }
            else {
                return false;
            }
        });
    }
    canUpdateNsByStr(nsStr, throwOnErr = false) {
        return __awaiter(this, void 0, void 0, function* () {
            var roleFact = this.getService().getRoles();
            var mapObjs = yield this.getNsMaps();
            for (var i in mapObjs) {
                var mapObj = mapObjs[i];
                if ((yield roleFact.canUpdate(yield mapObj.getRole(), false)) === true) {
                    var nsObj = yield mapObj.getNamespace();
                    var exStr = yield nsObj.getExtendedNamespace();
                    if (nsStr === exStr || nsStr.indexOf(exStr) === 0) {
                        return true;
                    }
                }
            }
            if (throwOnErr === true) {
                throw this.getMtm().newException("User is not allowed to update the namespace", 19559);
            }
            else {
                return false;
            }
        });
    }
    canUpdateSensitiveNsByStr(nsStr, throwOnErr = false) {
        return __awaiter(this, void 0, void 0, function* () {
            var roleFact = this.getService().getRoles();
            var mapObjs = yield this.getNsMaps();
            for (var i in mapObjs) {
                var mapObj = mapObjs[i];
                if ((yield roleFact.canUpdateSensitive(yield mapObj.getRole(), false)) === true) {
                    var nsObj = yield mapObj.getNamespace();
                    var exStr = yield nsObj.getExtendedNamespace();
                    if (nsStr === exStr || nsStr.indexOf(exStr) === 0) {
                        return true;
                    }
                }
            }
            if (throwOnErr === true) {
                throw this.getMtm().newException("User is not allowed to update sensitive in the namespace", 19562);
            }
            else {
                return false;
            }
        });
    }
    canCreateNsByStr(nsStr, throwOnErr = false) {
        return __awaiter(this, void 0, void 0, function* () {
            var roleFact = this.getService().getRoles();
            var mapObjs = yield this.getNsMaps();
            for (var i in mapObjs) {
                var mapObj = mapObjs[i];
                if ((yield roleFact.canCreate(yield mapObj.getRole(), false)) === true) {
                    var nsObj = yield mapObj.getNamespace();
                    var exStr = yield nsObj.getExtendedNamespace();
                    if (nsStr === exStr || nsStr.indexOf(exStr) === 0) {
                        return true;
                    }
                }
            }
            if (throwOnErr === true) {
                throw this.getMtm().newException("User is not allowed to create in the namespace", 19560);
            }
            else {
                return false;
            }
        });
    }
    canDeleteNsByStr(nsStr, throwOnErr = false) {
        return __awaiter(this, void 0, void 0, function* () {
            var roleFact = this.getService().getRoles();
            var mapObjs = yield this.getNsMaps();
            for (var i in mapObjs) {
                var mapObj = mapObjs[i];
                if ((yield roleFact.canDelete(yield mapObj.getRole(), false)) === true) {
                    var nsObj = yield mapObj.getNamespace();
                    var exStr = yield nsObj.getExtendedNamespace();
                    if (nsStr === exStr || nsStr.indexOf(exStr) === 0) {
                        return true;
                    }
                }
            }
            if (throwOnErr === true) {
                throw this.getMtm().newException("User is not allowed to delete in the namespace", 19561);
            }
            else {
                return false;
            }
        });
    }
    canReadMgmByStr(nsStr, throwOnErr = false) {
        return __awaiter(this, void 0, void 0, function* () {
            var roleFact = this.getService().getRoles();
            var mapObjs = yield this.getMgmMaps();
            for (var i in mapObjs) {
                var mapObj = mapObjs[i];
                if ((yield roleFact.canRead(yield mapObj.getRole(), false)) === true) {
                    var nsObj = yield mapObj.getNamespace();
                    var exStr = yield nsObj.getExtendedNamespace();
                    if (nsStr === exStr || nsStr.indexOf(exStr) === 0) {
                        return true;
                    }
                }
            }
            if (throwOnErr === true) {
                throw this.getMtm().newException("User is not allowed to read the management namespace", 19563);
            }
            else {
                return false;
            }
        });
    }
    canReadSensitiveMgmByStr(nsStr, throwOnErr = false) {
        return __awaiter(this, void 0, void 0, function* () {
            var roleFact = this.getService().getRoles();
            var mapObjs = yield this.getMgmMaps();
            for (var i in mapObjs) {
                var mapObj = mapObjs[i];
                if ((yield roleFact.canReadSensitive(yield mapObj.getRole(), false)) === true) {
                    var nsObj = yield mapObj.getNamespace();
                    var exStr = yield nsObj.getExtendedNamespace();
                    if (nsStr === exStr || nsStr.indexOf(exStr) === 0) {
                        return true;
                    }
                }
            }
            if (throwOnErr === true) {
                throw this.getMtm().newException("User is not allowed to read sensitive in the management namespace", 19564);
            }
            else {
                return false;
            }
        });
    }
    canExecuteMgmByStr(nsStr, throwOnErr = false) {
        return __awaiter(this, void 0, void 0, function* () {
            var roleFact = this.getService().getRoles();
            var mapObjs = yield this.getMgmMaps();
            for (var i in mapObjs) {
                var mapObj = mapObjs[i];
                if ((yield roleFact.canExecute(yield mapObj.getRole(), false)) === true) {
                    var nsObj = yield mapObj.getNamespace();
                    var exStr = yield nsObj.getExtendedNamespace();
                    if (nsStr === exStr || nsStr.indexOf(exStr) === 0) {
                        return true;
                    }
                }
            }
            if (throwOnErr === true) {
                throw this.getMtm().newException("User is not allowed to execute in the management namespace", 19565);
            }
            else {
                return false;
            }
        });
    }
    canUpdateMgmByStr(nsStr, throwOnErr = false) {
        return __awaiter(this, void 0, void 0, function* () {
            var roleFact = this.getService().getRoles();
            var mapObjs = yield this.getMgmMaps();
            for (var i in mapObjs) {
                var mapObj = mapObjs[i];
                if ((yield roleFact.canUpdate(yield mapObj.getRole(), false)) === true) {
                    var nsObj = yield mapObj.getNamespace();
                    var exStr = yield nsObj.getExtendedNamespace();
                    if (nsStr === exStr || nsStr.indexOf(exStr) === 0) {
                        return true;
                    }
                }
            }
            if (throwOnErr === true) {
                throw this.getMtm().newException("User is not allowed to update the management namespace", 19566);
            }
            else {
                return false;
            }
        });
    }
    canUpdateSensitiveMgmByStr(nsStr, throwOnErr = false) {
        return __awaiter(this, void 0, void 0, function* () {
            var roleFact = this.getService().getRoles();
            var mapObjs = yield this.getMgmMaps();
            for (var i in mapObjs) {
                var mapObj = mapObjs[i];
                if ((yield roleFact.canUpdateSensitive(yield mapObj.getRole(), false)) === true) {
                    var nsObj = yield mapObj.getNamespace();
                    var exStr = yield nsObj.getExtendedNamespace();
                    if (nsStr === exStr || nsStr.indexOf(exStr) === 0) {
                        return true;
                    }
                }
            }
            if (throwOnErr === true) {
                throw this.getMtm().newException("User is not allowed to update sensitive in the management namespace", 19567);
            }
            else {
                return false;
            }
        });
    }
    canCreateMgmByStr(nsStr, throwOnErr = false) {
        return __awaiter(this, void 0, void 0, function* () {
            var roleFact = this.getService().getRoles();
            var mapObjs = yield this.getMgmMaps();
            for (var i in mapObjs) {
                var mapObj = mapObjs[i];
                if ((yield roleFact.canCreate(yield mapObj.getRole(), false)) === true) {
                    var nsObj = yield mapObj.getNamespace();
                    var exStr = yield nsObj.getExtendedNamespace();
                    if (nsStr === exStr || nsStr.indexOf(exStr) === 0) {
                        return true;
                    }
                }
            }
            if (throwOnErr === true) {
                throw this.getMtm().newException("User is not allowed to create in the management namespace", 19568);
            }
            else {
                return false;
            }
        });
    }
    canDeleteMgmByStr(nsStr, throwOnErr = false) {
        return __awaiter(this, void 0, void 0, function* () {
            var roleFact = this.getService().getRoles();
            var mapObjs = yield this.getMgmMaps();
            for (var i in mapObjs) {
                var mapObj = mapObjs[i];
                if ((yield roleFact.canDelete(yield mapObj.getRole(), false)) === true) {
                    var nsObj = yield mapObj.getNamespace();
                    var exStr = yield nsObj.getExtendedNamespace();
                    if (nsStr === exStr || nsStr.indexOf(exStr) === 0) {
                        return true;
                    }
                }
            }
            if (throwOnErr === true) {
                throw this.getMtm().newException("User is not allowed to read the management namespace", 19569);
            }
            else {
                return false;
            }
        });
    }
}

class RoleMgmMaps extends Privileges {
    addMgmMap(nsObj, roleObj) {
        return __awaiter(this, void 0, void 0, function* () {
            if ((yield this.getMgmMapByNamespace(nsObj, false)) !== null) {
                throw this.getMtm().newException("Namespace already mapped");
            }
            var reqObj = yield this.newRequest();
            reqObj.setL1("RoleMgmMaps").setL2("Add");
            reqObj.addReq("user-guid", this.getGuid());
            reqObj.addReq("role-guid", roleObj.getGuid());
            reqObj.addReq("ns-guid", nsObj.getGuid());
            var apiObj = yield reqObj.getResponse(true);
            return yield this.getService().getRoleMgmMaps().getByApiObj(apiObj);
        });
    }
    getMgmMaps(cbObj = null, cbMethod = null, cbRef = null) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._mgmGuids === null) {
                var reqObj = yield this.newRequest();
                reqObj.setL1("RoleMgmMaps").setL2("Get").setL3("ByUser");
                reqObj.addReq("guid", this.getGuid());
                this._mgmGuids = yield reqObj.getResponse(true);
            }
            return yield this.getService().getRoleMgmMaps().getByGuids(this._mgmGuids, cbObj, cbMethod, cbRef);
        });
    }
    getMgmMapByNamespace(nsObj, throwOnErr = false) {
        return __awaiter(this, void 0, void 0, function* () {
            var rObjs = yield this.getMgmMaps();
            for (var i in rObjs) {
                var eObj = yield rObjs[i].getNamespace();
                if (eObj.getGuid() === nsObj.getGuid()) {
                    return rObjs[i];
                }
            }
            if (throwOnErr === true) {
                throw this.getMtm().newException("No management map matches the namespace");
            }
            else {
                return null;
            }
        });
    }
}

class RoleNsMaps extends RoleMgmMaps {
    addNsMap(nsObj, roleObj) {
        return __awaiter(this, void 0, void 0, function* () {
            if ((yield this.getNsMapByNamespace(nsObj, false)) !== null) {
                throw this.getMtm().newException("Namespace already mapped");
            }
            var reqObj = yield this.newRequest();
            reqObj.setL1("RoleNsMaps").setL2("Add");
            reqObj.addReq("user-guid", this.getGuid());
            reqObj.addReq("role-guid", roleObj.getGuid());
            reqObj.addReq("ns-guid", nsObj.getGuid());
            var apiObj = yield reqObj.getResponse(true);
            return yield this.getService().getRoleNsMaps().getByApiObj(apiObj);
        });
    }
    getNsMaps(cbObj = null, cbMethod = null, cbRef = null) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._nsGuids === null) {
                var reqObj = yield this.newRequest();
                reqObj.setL1("RoleNsMaps").setL2("Get").setL3("ByUser");
                reqObj.addReq("guid", this.getGuid());
                this._nsGuids = yield reqObj.getResponse(true);
            }
            return yield this.getService().getRoleNsMaps().getByGuids(this._nsGuids, cbObj, cbMethod, cbRef);
        });
    }
    getNsMapByNamespace(nsObj, throwOnErr = false) {
        return __awaiter(this, void 0, void 0, function* () {
            var rObjs = yield this.getNsMaps();
            for (var i in rObjs) {
                var eObj = yield rObjs[i].getNamespace();
                if (eObj.getGuid() === nsObj.getGuid()) {
                    return rObjs[i];
                }
            }
            if (throwOnErr === true) {
                throw this.getMtm().newException("No ns map matches the namespace");
            }
            else {
                return null;
            }
        });
    }
}

class Zstance extends RoleNsMaps {
    terminate(throwOnErr = false) {
        this.setTerm();
        this.getService().getUsers().clearCache(this);
    }
}

class Users extends BaseFactory {
    constructor() {
        super(...arguments);
        this._connGuid = null;
    }
    newObj() {
        var rObj = new Zstance();
        rObj.setService(this.getService());
        return rObj;
    }
    getByApiObj(apiObj) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof this._s[apiObj.guid] === "undefined") {
                this._s[apiObj.guid] = this.newObj();
                this._s[apiObj.guid].construct(apiObj);
            }
            yield this._s[apiObj.guid].waitInit();
            return this._s[apiObj.guid];
        });
    }
    getByConnection() {
        return __awaiter(this, void 0, void 0, function* () {
            //get self
            if (this._connGuid !== null) {
                return yield this.getByGuid(this._connGuid);
            }
            var reqObj = yield this.newRequest();
            reqObj.setL1("Users").setL2("Get").setL3("ByConnection");
            try {
                var apiObj = yield reqObj.getResponse(true);
                this._connGuid = apiObj.guid;
                return yield this.getByApiObj(apiObj);
            }
            catch (errObj) {
                this.callException(errObj);
                throw errObj;
            }
        });
    }
    getAll(cbObj = null, cbMethod = null, cbRef = null) {
        return __awaiter(this, void 0, void 0, function* () {
            var reqObj = yield this.newRequest();
            reqObj.setL1("Users").setL2("Get").setL3("All");
            try {
                var guids = yield reqObj.getResponse(true);
                return yield this.getByGuids(guids, cbObj, cbMethod, cbRef);
            }
            catch (errObj) {
                this.callException(errObj);
                throw errObj;
            }
        });
    }
    getByGuid(guid, cbObj = null, cbMethod = null, cbRef = null) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof this._s[guid] !== "undefined") {
                yield this._s[guid].waitInit();
                if (cbObj !== null && cbMethod !== null) {
                    cbObj[cbMethod](this._s[guid], cbRef);
                }
                return this._s[guid];
            }
            var reqObj = yield this.newRequest();
            reqObj.setL1("Users").setL2("Get").setL3("ByGuid");
            reqObj.addReq("guid", guid);
            try {
                var apiObj = yield reqObj.getResponse(true);
                var rObj = yield this.getByApiObj(apiObj);
                if (cbObj !== null && cbMethod !== null) {
                    cbObj[cbMethod](rObj, cbRef);
                }
                return rObj;
            }
            catch (errObj) {
                throw errObj;
            }
        });
    }
    addByEas(displayName) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                //only the user self can sign up, we need their EAS connection to get a key
                var ephKey = yield this.getMtm().get("DcEasApi").getEphemeralKeys().newReadUserPublic();
                var sigStr = "UNR--SIGNMEUP";
                var hash = yield this.getMtm().get("utilities").getHashing().getSHA().get256(sigStr);
                var reqObj = yield this.newRequest();
                reqObj.setL1("Users").setL2("Add").setL3("ByEAS").setL4("EphemeralKey");
                reqObj.addReq("displayName", displayName);
                yield ephKey.signRequest(reqObj, hash);
                var apiObj = yield reqObj.getResponse(true);
                return yield this.getByApiObj(apiObj);
            }
            catch (errObj) {
                this.callException(errObj);
                throw errObj;
            }
        });
    }
}

class Role extends Base {
    constructor() {
        super(...arguments);
        this._privGuids = null;
    }
    construct(apiObj) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.baseConstruct(apiObj);
            var storeObj = yield this.pubStore();
            storeObj.addUpdateCb(this, "ddsUpdateCb");
            this.setInit();
            return this;
        });
    }
    getLabel() {
        return this.getPubKey("label", true);
    }
    getDisplayName() {
        return this.getPubKey("displayName", true);
    }
    getDescription() {
        return this.getPubKey("desc", true);
    }
    getCreated() {
        return this.getPubKey("created", true);
    }
    getPrivilegeMaps(cbObj = null, cbMethod = null, cbRef = null) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._privGuids === null) {
                var reqObj = yield this.newRequest();
                reqObj.setL1("RolePrivilegeMaps").setL2("Get").setL3("ByRole");
                reqObj.addReq("guid", this.getGuid());
                this._privGuids = yield reqObj.getResponse(true);
            }
            var rObjs = [];
            for (let i in this._privGuids) {
                var rObj = yield this.getService().getRolePrivilegeMaps().getByGuid(this._privGuids[i], cbObj, cbMethod, cbRef);
                rObjs.push(rObj);
            }
            return rObjs;
        });
    }
    getPrivilegeMapByPrivilege(privObj, throwOnErr = false) {
        return __awaiter(this, void 0, void 0, function* () {
            var mapObjs = yield this.getPrivilegeMaps();
            for (let i in mapObjs) {
                var eObj = yield mapObjs[i].getPrivilege();
                if (eObj.getGuid() === privObj.getGuid()) {
                    return mapObjs[i];
                }
            }
            if (throwOnErr === true) {
                throw this.getMtm().newException("Privilege is not mapped");
            }
            else {
                return null;
            }
        });
    }
    ddsUpdateCb(storeObj) {
        this._privGuids = null;
    }
    terminate(throwOnErr = false) {
        this.setTerm();
        this.getService().getRoles().clearCache(this);
    }
}

class Roles extends BaseFactory {
    newObj() {
        var rObj = new Role();
        rObj.setService(this.getService());
        return rObj;
    }
    getByApiObj(apiObj) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof this._s[apiObj.guid] === "undefined") {
                this._s[apiObj.guid] = this.newObj();
                this._s[apiObj.guid].construct(apiObj);
            }
            yield this._s[apiObj.guid].waitInit();
            return this._s[apiObj.guid];
        });
    }
    getAll(cbObj = null, cbMethod = null, cbRef = null) {
        return __awaiter(this, void 0, void 0, function* () {
            var reqObj = yield this.newRequest();
            reqObj.setL1("Roles").setL2("Get").setL3("All");
            try {
                var guids = yield reqObj.getResponse(true);
                return yield this.getByGuids(guids, cbObj, cbMethod, cbRef);
            }
            catch (errObj) {
                this.callException(errObj);
                throw errObj;
            }
        });
    }
    getByGuid(guid, cbObj = null, cbMethod = null, cbRef = null) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof this._s[guid] !== "undefined") {
                yield this._s[guid].waitInit();
                if (cbObj !== null && cbMethod !== null) {
                    cbObj[cbMethod](this._s[guid], cbRef);
                }
                return this._s[guid];
            }
            var reqObj = yield this.newRequest();
            reqObj.setL1("Roles").setL2("Get").setL3("ByGuid");
            reqObj.addReq("guid", guid);
            try {
                var apiObj = yield reqObj.getResponse(true);
                var rObj = yield this.getByApiObj(apiObj);
                if (cbObj !== null && cbMethod !== null) {
                    cbObj[cbMethod](rObj, cbRef);
                }
                return rObj;
            }
            catch (errObj) {
                throw errObj;
            }
        });
    }
    getByLabel(label) {
        return __awaiter(this, void 0, void 0, function* () {
            for (let key of Object.keys(this._s)) {
                var eObj = this._s[key];
                yield eObj.waitInit();
                if (eObj.getLabel() == label) {
                    return eObj;
                }
            }
            var reqObj = yield this.newRequest();
            reqObj.setL1("Roles").setL2("Get").setL3("ByLabel");
            reqObj.addReq("label", label);
            try {
                var apiObj = yield reqObj.getResponse(true);
                var rObj = yield this.getByApiObj(apiObj);
                return rObj;
            }
            catch (errObj) {
                this.callException(errObj);
                throw errObj;
            }
        });
    }
    canRead(roleObj, throwOnErr = false) {
        return __awaiter(this, void 0, void 0, function* () {
            var privObj = yield this.getService().getPrivileges().getByLabel("read");
            var mapObj = yield roleObj.getPrivilegeMapByPrivilege(privObj, false);
            if (mapObj !== null) {
                return true;
            }
            else if (throwOnErr === true) {
                throw this.getMtm().newException("Role does not have the read privilige");
            }
            else {
                return false;
            }
        });
    }
    canReadSensitive(roleObj, throwOnErr = false) {
        return __awaiter(this, void 0, void 0, function* () {
            var privObj = yield this.getService().getPrivileges().getByLabel("read-sensitive");
            var mapObj = yield roleObj.getPrivilegeMapByPrivilege(privObj, false);
            if (mapObj !== null) {
                return true;
            }
            else if (throwOnErr === true) {
                throw this.getMtm().newException("Role does not have the read-sensitive privilige");
            }
            else {
                return false;
            }
        });
    }
    canExecute(roleObj, throwOnErr = false) {
        return __awaiter(this, void 0, void 0, function* () {
            var privObj = yield this.getService().getPrivileges().getByLabel("execute");
            var mapObj = yield roleObj.getPrivilegeMapByPrivilege(privObj, false);
            if (mapObj !== null) {
                return true;
            }
            else if (throwOnErr === true) {
                throw this.getMtm().newException("Role does not have the execute privilige");
            }
            else {
                return false;
            }
        });
    }
    canUpdate(roleObj, throwOnErr = false) {
        return __awaiter(this, void 0, void 0, function* () {
            var privObj = yield this.getService().getPrivileges().getByLabel("update");
            var mapObj = yield roleObj.getPrivilegeMapByPrivilege(privObj, false);
            if (mapObj !== null) {
                return true;
            }
            else if (throwOnErr === true) {
                throw this.getMtm().newException("Role does not have the update privilige");
            }
            else {
                return false;
            }
        });
    }
    canUpdateSensitive(roleObj, throwOnErr = false) {
        return __awaiter(this, void 0, void 0, function* () {
            var privObj = yield this.getService().getPrivileges().getByLabel("update-sensitive");
            var mapObj = yield roleObj.getPrivilegeMapByPrivilege(privObj, false);
            if (mapObj !== null) {
                return true;
            }
            else if (throwOnErr === true) {
                throw this.getMtm().newException("Role does not have the update-sensitive privilige");
            }
            else {
                return false;
            }
        });
    }
    canCreate(roleObj, throwOnErr = false) {
        return __awaiter(this, void 0, void 0, function* () {
            var privObj = yield this.getService().getPrivileges().getByLabel("create");
            var mapObj = yield roleObj.getPrivilegeMapByPrivilege(privObj, false);
            if (mapObj !== null) {
                return true;
            }
            else if (throwOnErr === true) {
                throw this.getMtm().newException("Role does not have the create privilige");
            }
            else {
                return false;
            }
        });
    }
    canDelete(roleObj, throwOnErr = false) {
        return __awaiter(this, void 0, void 0, function* () {
            var privObj = yield this.getService().getPrivileges().getByLabel("delete");
            var mapObj = yield roleObj.getPrivilegeMapByPrivilege(privObj, false);
            if (mapObj !== null) {
                return true;
            }
            else if (throwOnErr === true) {
                throw this.getMtm().newException("Role does not have the delete privilige");
            }
            else {
                return false;
            }
        });
    }
}

class Privilege extends Base {
    construct(apiObj) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.baseConstruct(apiObj);
            this.setInit();
            return this;
        });
    }
    getLabel() {
        return this.getPubKey("label", true);
    }
    getDisplayName() {
        return this.getPubKey("displayName", true);
    }
    getDescription() {
        return this.getPubKey("desc", true);
    }
    getCreated() {
        return this.getPubKey("created", true);
    }
    terminate(throwOnErr = false) {
        this.setTerm();
        this.getService().getPrivileges().clearCache(this);
    }
}

class Privileges$1 extends BaseFactory {
    newObj() {
        var rObj = new Privilege();
        rObj.setService(this.getService());
        return rObj;
    }
    getByApiObj(apiObj) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof this._s[apiObj.guid] === "undefined") {
                this._s[apiObj.guid] = this.newObj();
                this._s[apiObj.guid].construct(apiObj);
            }
            yield this._s[apiObj.guid].waitInit();
            return this._s[apiObj.guid];
        });
    }
    getAll(cbObj = null, cbMethod = null, cbRef = null) {
        return __awaiter(this, void 0, void 0, function* () {
            var reqObj = yield this.newRequest();
            reqObj.setL1("Privileges").setL2("Get").setL3("All");
            try {
                var guids = yield reqObj.getResponse(true);
                return yield this.getByGuids(guids, cbObj, cbMethod, cbRef);
            }
            catch (errObj) {
                this.callException(errObj);
                throw errObj;
            }
        });
    }
    getByGuid(guid, cbObj = null, cbMethod = null, cbRef = null) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof this._s[guid] !== "undefined") {
                yield this._s[guid].waitInit();
                if (cbObj !== null && cbMethod !== null) {
                    cbObj[cbMethod](this._s[guid], cbRef);
                }
                return this._s[guid];
            }
            var reqObj = yield this.newRequest();
            reqObj.setL1("Privileges").setL2("Get").setL3("ByGuid");
            reqObj.addReq("guid", guid);
            try {
                var apiObj = yield reqObj.getResponse(true);
                var rObj = yield this.getByApiObj(apiObj);
                if (cbObj !== null && cbMethod !== null) {
                    cbObj[cbMethod](rObj, cbRef);
                }
                return rObj;
            }
            catch (errObj) {
                throw errObj;
            }
        });
    }
    getByLabel(label) {
        return __awaiter(this, void 0, void 0, function* () {
            for (let key of Object.keys(this._s)) {
                var eObj = this._s[key];
                yield eObj.waitInit();
                if (eObj.getLabel() == label) {
                    return eObj;
                }
            }
            var reqObj = yield this.newRequest();
            reqObj.setL1("Privileges").setL2("Get").setL3("ByLabel");
            reqObj.addReq("label", label);
            try {
                var apiObj = yield reqObj.getResponse(true);
                var rObj = yield this.getByApiObj(apiObj);
                return rObj;
            }
            catch (errObj) {
                this.callException(errObj);
                throw errObj;
            }
        });
    }
}

class Alpha$1 extends Base {
    constructor() {
        super(...arguments);
        this._childGuids = null;
        this._mgmMaps = null;
        this._nsMaps = null;
    }
    getParent() { }
    ;
    construct(apiObj) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.baseConstruct(apiObj);
            var storeObj = yield this.pubStore();
            storeObj.addUpdateCb(this, "ddsUpdateCb");
            this.setInit();
            return this;
        });
    }
    getNamespace() {
        return this.getPubKey("ns", true);
    }
    getExtendedNamespace() {
        return __awaiter(this, void 0, void 0, function* () {
            var pObj = yield this.getParent();
            if (pObj !== null) {
                return (yield pObj.getExtendedNamespace()) + "." + this.getNamespace();
            }
            else {
                return this.getNamespace();
            }
        });
    }
    isSystem() {
        return this.getPubKey("isSystem", true);
    }
    setSystem(bool) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (this.isSystem() !== bool) {
                    var reqObj = yield this.getService().newRequest();
                    reqObj.setL1("Namespaces").setL2("Set").setL3("System");
                    reqObj.addReq("guid", this.getGuid());
                    reqObj.addReq("isSystem", bool);
                    var waitUp = this.getPubUpdateWait();
                    try {
                        yield reqObj.getResponse(true);
                        yield waitUp;
                    }
                    catch (errObj) {
                        this.pubUpdateClear(waitUp);
                        throw errObj;
                    }
                }
                return this;
            }
            catch (errObj) {
                this.callException(errObj);
                throw errObj;
            }
        });
    }
    isImmutable() {
        return this.getPubKey("isImmutable", true);
    }
    setImmutable(bool) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                if (this.isImmutable() !== bool) {
                    var reqObj = yield this.getService().newRequest();
                    reqObj.setL1("Namespaces").setL2("Set").setL3("Immutable");
                    reqObj.addReq("guid", this.getGuid());
                    reqObj.addReq("isImmutable", bool);
                    var waitUp = this.getPubUpdateWait();
                    try {
                        yield reqObj.getResponse(true);
                        yield waitUp;
                    }
                    catch (errObj) {
                        this.pubUpdateClear(waitUp);
                        throw errObj;
                    }
                }
                return this;
            }
            catch (errObj) {
                this.callException(errObj);
                throw errObj;
            }
        });
    }
    getCreated() {
        return this.getPubKey("created", true);
    }
}

class DdsSync$1 extends Alpha$1 {
    ddsUpdateCb(storeObj) {
        this._childGuids = null;
        this._mgmMaps = null;
        this._nsMaps = null;
    }
}

class Namespaces extends DdsSync$1 {
    getParent() {
        return __awaiter(this, void 0, void 0, function* () {
            var pGuid = this.getPubKey("pGuid", true);
            if (pGuid !== null) {
                return yield this.getService().getNamespaces().getByGuid(pGuid);
            }
            else {
                return null;
            }
        });
    }
    addChild(ns) {
        return __awaiter(this, void 0, void 0, function* () {
            if ((yield this.getChildByNamespace(ns, false)) !== null) {
                throw this.getMtm().newException("Cannot add child, namespace exists");
            }
            var reqObj = yield this.newRequest();
            reqObj.setL1("Namespaces").setL2("Add");
            reqObj.addReq("guid", this.getGuid());
            reqObj.addReq("ns", ns);
            var apiObj = yield reqObj.getResponse(true);
            return yield this.getService().getNamespaces().getByApiObj(apiObj);
        });
    }
    getChildren(cbObj = null, cbMethod = null, cbRef = null) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._childGuids === null) {
                var reqObj = yield this.newRequest();
                reqObj.setL1("Namespaces").setL2("Get").setL3("Children").setL4("ByGuid");
                reqObj.addReq("guid", this.getGuid());
                this._childGuids = yield reqObj.getResponse(true);
            }
            return yield this.getService().getNamespaces().getByGuids(this._childGuids, cbObj, cbMethod, cbRef);
        });
    }
    getChildByNamespace(ns, throwOnErr = false) {
        return __awaiter(this, void 0, void 0, function* () {
            var rObjs = yield this.getChildren();
            for (var i in rObjs) {
                var rObj = rObjs[i];
                if (rObj.getNamespace() === ns) {
                    return rObj;
                }
            }
            if (throwOnErr === true) {
                throw this.getMtm().newException("Child namespace does not exist");
            }
            else {
                return null;
            }
        });
    }
}

class RoleMgmMaps$1 extends Namespaces {
    getMgmMaps(cbObj = null, cbMethod = null, cbRef = null) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._mgmMaps === null) {
                var reqObj = yield this.newRequest();
                reqObj.setL1("RoleMgmMaps").setL2("Get").setL3("ByNamespace");
                reqObj.addReq("guid", this.getGuid());
                this._mgmMaps = yield reqObj.getResponse(true);
            }
            return yield this.getService().getRoleMgmMaps().getByGuids(this._mgmMaps, cbObj, cbMethod, cbRef);
        });
    }
}

class RoleNsMaps$1 extends RoleMgmMaps$1 {
    getNsMaps(cbObj = null, cbMethod = null, cbRef = null) {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._nsMaps === null) {
                var reqObj = yield this.newRequest();
                reqObj.setL1("RoleNsMaps").setL2("Get").setL3("ByNamespace");
                reqObj.addReq("guid", this.getGuid());
                this._nsMaps = yield reqObj.getResponse(true);
            }
            return yield this.getService().getRoleNsMaps().getByGuids(this._nsMaps, cbObj, cbMethod, cbRef);
        });
    }
}

class Zstance$1 extends RoleNsMaps$1 {
    delete() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isDel() === false) {
                var reqObj = yield this.getService().newRequest();
                reqObj.setL1("Namespaces").setL2("Delete").setL3("ByGuid");
                reqObj.addReq("guid", this.getGuid());
                var waitDel = this.getPubDeleteWait();
                try {
                    yield reqObj.getResponse(true);
                    this.setDel();
                    yield waitDel;
                    this.terminate();
                }
                catch (errObj) {
                    this.pubDeleteClear();
                    throw errObj;
                }
            }
            return this;
        });
    }
    terminate(throwOnErr = false) {
        this.setTerm();
        this.getService().getNamespaces().clearCache(this);
    }
}

class Namespaces$1 extends BaseFactory {
    constructor() {
        super(...arguments);
        this._rootGuid = null;
    }
    newObj() {
        var rObj = new Zstance$1();
        rObj.setService(this.getService());
        return rObj;
    }
    getByApiObj(apiObj) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof this._s[apiObj.guid] === "undefined") {
                this._s[apiObj.guid] = this.newObj();
                this._s[apiObj.guid].construct(apiObj);
            }
            yield this._s[apiObj.guid].waitInit();
            return this._s[apiObj.guid];
        });
    }
    getRoot() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._rootGuid !== null) {
                return yield this.getByGuid(this._rootGuid);
            }
            var reqObj = yield this.newRequest();
            reqObj.setL1("Namespaces").setL2("Get").setL3("Root");
            try {
                var apiObj = yield reqObj.getResponse(true);
                this._rootGuid = apiObj.guid;
                return yield this.getByApiObj(apiObj);
            }
            catch (errObj) {
                this.callException(errObj);
                throw errObj;
            }
        });
    }
    getByGuid(guid, cbObj = null, cbMethod = null, cbRef = null) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof this._s[guid] !== "undefined") {
                yield this._s[guid].waitInit();
                if (cbObj !== null && cbMethod !== null) {
                    cbObj[cbMethod](this._s[guid], cbRef);
                }
                return this._s[guid];
            }
            var reqObj = yield this.newRequest();
            reqObj.setL1("Namespaces").setL2("Get").setL3("ByGuid");
            reqObj.addReq("guid", guid);
            try {
                var apiObj = yield reqObj.getResponse(true);
                var rObj = yield this.getByApiObj(apiObj);
                if (cbObj !== null && cbMethod !== null) {
                    cbObj[cbMethod](rObj, cbRef);
                }
                return rObj;
            }
            catch (errObj) {
                throw errObj;
            }
        });
    }
}

class Alpha$2 extends Base {
    construct(apiObj) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.baseConstruct(apiObj);
            this.setInit();
            return this;
        });
    }
    getNamespace() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.getService().getNamespaces().getByGuid(this.getPubKey("nsGuid", true));
        });
    }
    getUser() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.getService().getUsers().getByGuid(this.getPubKey("userGuid", true));
        });
    }
    getRole() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.getService().getRoles().getByGuid(this.getPubKey("roleGuid", true));
        });
    }
    setRole(roleObj) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                var curObj = yield this.getRole();
                if (curObj.getGuid() !== roleObj.getGuid()) {
                    var reqObj = yield this.getService().newRequest();
                    reqObj.setL1("RoleNsMaps").setL2("Set").setL3("Role");
                    reqObj.addReq("guid", this.getGuid());
                    reqObj.addReq("role-guid", roleObj.getGuid());
                    var waitUp = this.getPubUpdateWait();
                    try {
                        yield reqObj.getResponse(true);
                        yield waitUp;
                    }
                    catch (errObj) {
                        this.pubUpdateClear(waitUp);
                        throw errObj;
                    }
                }
                return this;
            }
            catch (errObj) {
                this.callException(errObj);
                throw errObj;
            }
        });
    }
    getCreated() {
        return this.getPubKey("created", true);
    }
}

class DdsSync$2 extends Alpha$2 {
    ddsUpdateCb(storeObj) {
    }
}

class Zstance$2 extends DdsSync$2 {
    delete() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isDel() === false) {
                var reqObj = yield this.getService().newRequest();
                reqObj.setL1("RoleNsMaps").setL2("Delete").setL3("ByGuid");
                reqObj.addReq("guid", this.getGuid());
                var waitDel = this.getPubDeleteWait();
                try {
                    yield reqObj.getResponse(true);
                    this.setDel();
                    yield waitDel;
                    this.terminate();
                }
                catch (errObj) {
                    this.pubDeleteClear();
                    throw errObj;
                }
            }
            return this;
        });
    }
    terminate(throwOnErr = false) {
        this.setTerm();
        this.getService().getRoleNsMaps().clearCache(this);
    }
}

class RoleNsMaps$2 extends BaseFactory {
    newObj() {
        var rObj = new Zstance$2();
        rObj.setService(this.getService());
        return rObj;
    }
    getByApiObj(apiObj) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof this._s[apiObj.guid] === "undefined") {
                this._s[apiObj.guid] = this.newObj();
                this._s[apiObj.guid].construct(apiObj);
            }
            yield this._s[apiObj.guid].waitInit();
            return this._s[apiObj.guid];
        });
    }
    getByGuid(guid, cbObj = null, cbMethod = null, cbRef = null) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof this._s[guid] !== "undefined") {
                yield this._s[guid].waitInit();
                if (cbObj !== null && cbMethod !== null) {
                    cbObj[cbMethod](this._s[guid], cbRef);
                }
                return this._s[guid];
            }
            var reqObj = yield this.newRequest();
            reqObj.setL1("RoleNsMaps").setL2("Get").setL3("ByGuid");
            reqObj.addReq("guid", guid);
            try {
                var apiObj = yield reqObj.getResponse(true);
                var rObj = yield this.getByApiObj(apiObj);
                if (cbObj !== null && cbMethod !== null) {
                    cbObj[cbMethod](rObj, cbRef);
                }
                return rObj;
            }
            catch (errObj) {
                throw errObj;
            }
        });
    }
}

class Alpha$3 extends Base {
    construct(apiObj) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.baseConstruct(apiObj);
            this.setInit();
            return this;
        });
    }
    getNamespace() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.getService().getNamespaces().getByGuid(this.getPubKey("nsGuid", true));
        });
    }
    getUser() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.getService().getUsers().getByGuid(this.getPubKey("userGuid", true));
        });
    }
    getRole() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.getService().getRoles().getByGuid(this.getPubKey("roleGuid", true));
        });
    }
    setRole(roleObj) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                var curObj = yield this.getRole();
                if (curObj.getGuid() !== roleObj.getGuid()) {
                    var reqObj = yield this.getService().newRequest();
                    reqObj.setL1("RoleMgmMaps").setL2("Set").setL3("Role");
                    reqObj.addReq("guid", this.getGuid());
                    reqObj.addReq("role-guid", roleObj.getGuid());
                    var waitUp = this.getPubUpdateWait();
                    try {
                        yield reqObj.getResponse(true);
                        yield waitUp;
                    }
                    catch (errObj) {
                        this.pubUpdateClear(waitUp);
                        throw errObj;
                    }
                }
                return this;
            }
            catch (errObj) {
                this.callException(errObj);
                throw errObj;
            }
        });
    }
    getCreated() {
        return this.getPubKey("created", true);
    }
}

class DdsSync$3 extends Alpha$3 {
    ddsUpdateCb(storeObj) {
    }
}

class Zstance$3 extends DdsSync$3 {
    delete() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this.isDel() === false) {
                var reqObj = yield this.getService().newRequest();
                reqObj.setL1("RoleMgmMaps").setL2("Delete").setL3("ByGuid");
                reqObj.addReq("guid", this.getGuid());
                var waitDel = this.getPubDeleteWait();
                try {
                    yield reqObj.getResponse(true);
                    this.setDel();
                    yield waitDel;
                    this.terminate();
                }
                catch (errObj) {
                    this.pubDeleteClear();
                    throw errObj;
                }
            }
            return this;
        });
    }
    terminate(throwOnErr = false) {
        this.setTerm();
        this.getService().getRoleMgmMaps().clearCache(this);
    }
}

class RoleMgmMaps$2 extends BaseFactory {
    newObj() {
        var rObj = new Zstance$3();
        rObj.setService(this.getService());
        return rObj;
    }
    getByApiObj(apiObj) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof this._s[apiObj.guid] === "undefined") {
                this._s[apiObj.guid] = this.newObj();
                this._s[apiObj.guid].construct(apiObj);
            }
            yield this._s[apiObj.guid].waitInit();
            return this._s[apiObj.guid];
        });
    }
    getByGuid(guid, cbObj = null, cbMethod = null, cbRef = null) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof this._s[guid] !== "undefined") {
                yield this._s[guid].waitInit();
                if (cbObj !== null && cbMethod !== null) {
                    cbObj[cbMethod](this._s[guid], cbRef);
                }
                return this._s[guid];
            }
            var reqObj = yield this.newRequest();
            reqObj.setL1("RoleMgmMaps").setL2("Get").setL3("ByGuid");
            reqObj.addReq("guid", guid);
            try {
                var apiObj = yield reqObj.getResponse(true);
                var rObj = yield this.getByApiObj(apiObj);
                if (cbObj !== null && cbMethod !== null) {
                    cbObj[cbMethod](rObj, cbRef);
                }
                return rObj;
            }
            catch (errObj) {
                throw errObj;
            }
        });
    }
}

class RolePrivilegeMap extends Base {
    construct(apiObj) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.baseConstruct(apiObj);
            this.setInit();
            return this;
        });
    }
    getRole() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.getService().getRoles().getByGuid(this.getPubKey("roleGuid", true));
        });
    }
    getPrivilege() {
        return __awaiter(this, void 0, void 0, function* () {
            return yield this.getService().getPrivileges().getByGuid(this.getPubKey("privGuid", true));
        });
    }
    getCreated() {
        return this.getPubKey("created", true);
    }
    terminate(throwOnErr = false) {
        this.setTerm();
        this.getService().getRolePriviligeMaps().clearCache(this);
    }
}

class RolePrivilegeMaps extends BaseFactory {
    newObj() {
        var rObj = new RolePrivilegeMap();
        rObj.setService(this.getService());
        return rObj;
    }
    getByApiObj(apiObj) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof this._s[apiObj.guid] === "undefined") {
                this._s[apiObj.guid] = this.newObj();
                this._s[apiObj.guid].construct(apiObj);
            }
            yield this._s[apiObj.guid].waitInit();
            return this._s[apiObj.guid];
        });
    }
    getByGuid(guid, cbObj = null, cbMethod = null, cbRef = null) {
        return __awaiter(this, void 0, void 0, function* () {
            if (typeof this._s[guid] !== "undefined") {
                yield this._s[guid].waitInit();
                if (cbObj !== null && cbMethod !== null) {
                    cbObj[cbMethod](this._s[guid], cbRef);
                }
                return this._s[guid];
            }
            var reqObj = yield this.newRequest();
            reqObj.setL1("RolePrivilegeMaps").setL2("Get").setL3("ByGuid");
            reqObj.addReq("guid", guid);
            try {
                var apiObj = yield reqObj.getResponse(true);
                var rObj = yield this.getByApiObj(apiObj);
                if (cbObj !== null && cbMethod !== null) {
                    cbObj[cbMethod](rObj, cbRef);
                }
                return rObj;
            }
            catch (errObj) {
                throw errObj;
            }
        });
    }
}

class DcUnrApi {
    constructor(_dcScc, _dcEasApi) {
        this._dcScc = _dcScc;
        this._dcEasApi = _dcEasApi;
        this._connFact = null;
        this._uFact = null;
        this._nsFact = null;
        this._nrmFact = null;
        this._nmmFact = null;
        this._rpmFact = null;
        this._roleFact = null;
        this._privFact = null;
        this._ddsClient = null;
        this._dcScc.getMtm().set("DcUnrApi", this);
    }
    getDcScc() {
        return this._dcScc;
    }
    getMtm() {
        return this.getDcScc().getMtm();
    }
    getLogging() {
        return this.getDcScc().getLogging();
    }
    getConnection() {
        if (this._connFact === null) {
            this._connFact = new Connection();
            this._connFact.setService(this);
            this._connFact.setExceptionCb(this.getLogging(), "exception");
        }
        return this._connFact;
    }
    getRoleNsMaps() {
        if (this._nrmFact === null) {
            this._nrmFact = new RoleNsMaps$2();
            this._nrmFact.setService(this);
            this._nrmFact.setExceptionCb(this.getLogging(), "exception");
        }
        return this._nrmFact;
    }
    getRoleMgmMaps() {
        if (this._nmmFact === null) {
            this._nmmFact = new RoleMgmMaps$2();
            this._nmmFact.setService(this);
            this._nmmFact.setExceptionCb(this.getLogging(), "exception");
        }
        return this._nmmFact;
    }
    getRolePrivilegeMaps() {
        if (this._rpmFact === null) {
            this._rpmFact = new RolePrivilegeMaps();
            this._rpmFact.setService(this);
            this._rpmFact.setExceptionCb(this.getLogging(), "exception");
        }
        return this._rpmFact;
    }
    getNamespaces() {
        if (this._nsFact === null) {
            this._nsFact = new Namespaces$1();
            this._nsFact.setService(this);
            this._nsFact.setExceptionCb(this.getLogging(), "exception");
        }
        return this._nsFact;
    }
    getPrivileges() {
        if (this._privFact === null) {
            this._privFact = new Privileges$1();
            this._privFact.setService(this);
            this._privFact.setExceptionCb(this.getLogging(), "exception");
        }
        return this._privFact;
    }
    getRoles() {
        if (this._roleFact === null) {
            this._roleFact = new Roles();
            this._roleFact.setService(this);
            this._roleFact.setExceptionCb(this.getLogging(), "exception");
        }
        return this._roleFact;
    }
    getUsers() {
        if (this._uFact === null) {
            this._uFact = new Users();
            this._uFact.setService(this);
            this._uFact.setExceptionCb(this.getLogging(), "exception");
        }
        return this._uFact;
    }
    getDdsClient() {
        return __awaiter(this, void 0, void 0, function* () {
            if (this._ddsClient === null) {
                var ddsHost = "dc-wss-nlb.datacamo.net";
                var ddsPort = 28106;
                var ddsSsl = true;
                this._ddsClient = this.getMtm().get("DcDdsApi").getClients().getV1(ddsHost, ddsPort, ddsSsl);
                yield this._ddsClient.initialize();
            }
            return this._ddsClient;
        });
    }
    newRequest() {
        return __awaiter(this, void 0, void 0, function* () {
            var cObj = yield this.getConnection().getClient();
            var cObj = yield cObj.isAuth();
            return cObj.newMessage();
        });
    }
}
DcUnrApi.ɵprov = ɵɵdefineInjectable({ factory: function DcUnrApi_Factory() { return new DcUnrApi(ɵɵinject(DcScc), ɵɵinject(DcEasApi)); }, token: DcUnrApi, providedIn: "root" });
DcUnrApi.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
DcUnrApi.ctorParameters = () => [
    { type: DcScc },
    { type: DcEasApi }
];

/*
 * Public API Surface of dc-unr-api
 */

/**
 * Generated bundle index. Do not edit.
 */

export { DcUnrApi };
//# sourceMappingURL=dc-unr-api.js.map
